﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using UnityEngine.SceneManagement;  // シーンの切り替えに使用
using UnityEngine.UI;  // UIを使用するのに使用

//////////////////////////////////////////////////////////////////
//【ブロック全体・パズル自体の管理に使用するクラス】
// スタート時のブロックの配置、現在移動可能な位置番号の管理、パズルの完了判定処理
//////////////////////////////////////////////////////////////////
public class BlockManager : MonoBehaviour
{
    //【ブロックの位置番号】
    //　１　２　３
    //　４　５　６
    //　７　８　９

    /////////////////////////////////
    // 【変数の設定】
    /////////////////////////////////

    //【定数の設定】（レベル設計で変更可能）
    const int MAX_LEVEL = 17;  // MAXのレベル番号

    //【Unity Editorから設定するPublic変数】
    public BlockController[] class_BlockController;  
          // 各ブロックのBlockControllerの関数呼び出し用
          //（配列データはブロック番号が１から８の順番に設定されたBlockControllerクラスを設定）
    public GameObject imagePicture;  // パズル完成時に表示する完成写真
    public GameObject textComplete;  // パズル完成時に表示するCompletedのUIテキスト
    public Text text_Level;  // 現在のレベル番号を表示するUIテキスト

    //【このプログラム内で使用するPrivate変数】
    private int level_Num;  // 現在のレベル番号
    private int[] posNum_Initial;  // 各レベルのスタート時でのブロックの配置を設定（レベル設計）
          //（配列のデータはブロック番号１から８の順番にブロックの位置番号を設定）
          //（posNum_Initial[0]がブロック番号１の位置番号、
          //  posNum_Initial[1]がブロック番号２の位置番号、...）
    private int posNum_MovePossible = 9;  // 現在ブロックが移動可能な位置番合
          //（初期値=9、どのレベルでも最初は位置番号9が移動可能）
          //（ブロックが移動可能な位置番合はどのブロックにも共通なのでBlockManagerで管理）


    /////////////////////////////////
    // 【関数の設定】
    /////////////////////////////////

    //【ゲーム開始時、Start関数より前に１回だけ動く関数】
    //（Unityの標準関数）
    void Awake()
    {
        // 保存されている現在のレベル番号を取得
        level_Num = PlayerPrefs.GetInt("LEVEL_NUM", 1);

        // 現在のレベルに対するスタート時でのブロックの配置を取得
        //（レベル設計：各レベルでのパズルの配置）
        switch (level_Num)
        {
            case 1:
                posNum_Initial = new int[] { 1, 5, 2, 4, 8, 3, 7, 6 };
                break;
            case 2:
                posNum_Initial = new int[] { 2, 4, 3, 1, 5, 6, 7, 8 };
                break;
            case 3:
                posNum_Initial = new int[] { 4, 1, 2, 5, 8, 3, 7, 6 };
                break;
            case 4:
                posNum_Initial = new int[] { 2, 3, 6, 1, 4, 8, 7, 5 };
                break;
            case 5:
                posNum_Initial = new int[] { 5, 1, 3, 2, 7, 6, 4, 8 };
                break;
            case 6:
                posNum_Initial = new int[] { 4, 3, 5, 2, 1, 6, 7, 8 };
                break;
            case 7:
                posNum_Initial = new int[] { 2, 6, 5, 1, 7, 3, 4, 8 };
                break;
            case 8:
                posNum_Initial = new int[] { 4, 1, 3, 7, 2, 8, 6, 5 };
                break;
            case 9:
                posNum_Initial = new int[] { 3, 6, 8, 1, 2, 5, 4, 7 };
                break;
            case 10:
                posNum_Initial = new int[] { 3, 6, 8, 1, 2, 5, 4, 7 };
                break;
            case 11:
                posNum_Initial = new int[] { 4, 1, 3, 7, 2, 6, 5, 8 };
                break;
            case 12:
                posNum_Initial = new int[] { 5, 4, 1, 7, 8, 2, 6, 3 };
                break;
            case 13:
                posNum_Initial = new int[] { 5, 1, 3, 4, 6, 8, 7, 2 };
                break;
            case 14:
                posNum_Initial = new int[] { 4, 5, 1, 7, 6, 2, 8, 3 };
                break;
            case 15:
                posNum_Initial = new int[] { 4, 2, 3, 5, 6, 8, 7, 1 };
                break;
            case 16:
                posNum_Initial = new int[] { 4, 2, 5, 7, 6, 1, 8, 3 };
                break;
            case 17:
                posNum_Initial = new int[] { 4, 5, 6, 2, 3, 8, 7, 1 };
                break;
        }
    }


    //【ゲーム開始時に１回だけ動く関数】
    //（Unityの標準関数）
    void Start()
    {
        // スタート時に現在のレベル番号をUIに表示する
        text_Level.text = level_Num.ToString();
    }


    //【各ブロックが自分のブロック番号を引数としてスタート時の位置番号を取得する関数】
    public int OnGet_PosNum_Initial(int tmp_Block_Num)
    {
        return posNum_Initial[tmp_Block_Num - 1];
    }


    //【現在ブロックが移動可能な位置番合を返す関数】
    public int OnGet_PosNum_MovePossible()
    {
        return posNum_MovePossible;
    }


    //【次の移動可能な位置番号を今移動したブロックがセットする関数】
    public void OnSet_PosNum_MovePossible(int next_PosNum_MovePossible)
    {
        posNum_MovePossible = next_PosNum_MovePossible;
    }


    //【パズル完了判定処理を実行する関数】
    //（ブロックの移動が終了したタイミングで毎回呼ばれる）
    //（完了判定を行っているのは各ブロックで、BlockManagerはそれをとりまとめる）
    public void Check_Complete()
    {
        // 全ブロックのBlockControllerに対して処理を実行する
        foreach(BlockController tmp_Class in class_BlockController)
        {
            // 各ブロックのBlockControllerの完了判定処理関数を呼ぶ
            if (!tmp_Class.OnGet_FinishStatus())
            {
                // もし１つでも完了していないブロックがあれば完了判定処理は中止
                //（まだ全体としてパズルが完成していないことになるので）
                // returnでこの関数を抜けて終了する
                return;
            }
        }

        // 上のforeach文で全ブロックの完了判定を通過した場合
        //（パズルが完成したこととなるので、完了処理を実行する）

        // 完成版の写真を表示
        imagePicture.SetActive(true);
        // Completedの文字を表示
        textComplete.SetActive(true);
            
        // レベル番号がMAXのレベル番号より小さければ、次のレベルにする
        if (level_Num < MAX_LEVEL) level_Num++;
        // 次のレベル番号を保存する
        PlayerPrefs.SetInt("LEVEL_NUM", level_Num);           
    }


    //【リスタートボタン処理の実行関数】
    public void OnClick_Restart()
    {
        // ゲームシーンを再起動する
        SceneManager.LoadScene("GameMain");
    }


    //【リセットボタン処理の実行関数】
    public void OnClick_Reset()
    {
        // 現在のレベル番号を１で保存する
        PlayerPrefs.SetInt("LEVEL_NUM", 1);
        // ゲームシーンを再起動する
        SceneManager.LoadScene("GameMain");
    }
}
